


//
// 
//  File that implements GMM for MRW
//
//


#include "lastwave.h"
#include "signals.h"

#define EULER .577215664901

static LWFLOAT lntau;
static SIGNAL m = NULL;
static SIGNAL m2 = NULL;
static LWFLOAT var2;
static SIGNAL theLags = NULL;

/* The GMM Vector */
#define  GMMV_Var2  0
#define  GMMV_VarLn  -1
#define  GMMV_AutoCorLn  1 /* !!!!!!! Must be the last one and must be right after GMMV_VarLn */ 

/* The Parameters  */
#define  GMMP_Sigma2  0
#define  GMMP_Lambda2  1
#define  GMMP_LnT  2
#define  GMMP_Last  3


/* Ln epsilon theoretical variance */
static LWFLOAT VarianceLnEps(SIGNAL params)
{
  return(M_PI*M_PI/8);
}

/* Ln epsilon theoretical order 4 centered moment */
static LWFLOAT M4LnEps(SIGNAL params)
{
  return(M_PI*M_PI*M_PI*M_PI*7./64.);
}

/* Ln epsilon theoretical order 3 centered moment */
static LWFLOAT M3LnEps(SIGNAL params)
{
  return(-2.1036);
}

/* Mean of epsilon^2 * (centered_ln(epsilon)) */
static LWFLOAT MeanEps2lnEps(SIGNAL params)
{
  return(1.0);
}

/* Mean of epsilon^2 * (centered_ln(epsilon)^2) */
static LWFLOAT MeanEps2lnEps2(SIGNAL params)
{
  return(1.2337);
}

/* omega theoretical auto-correlation */
// ??????? A CHANGER 
// LWFLOAT CovLnSqrtVol (int m1, int n1, int m2, int n2, LWFLOAT lambda2, LWFLOAT T)
static LWFLOAT AutoCovarianceOm(SIGNAL params,int lag)
{
  LWFLOAT lambda2 = params->Y[GMMP_Lambda2];
  LWFLOAT lnT = params->Y[GMMP_LnT];
  
  lag = (lag<0 ? -lag : lag);
  
  switch(lag) {
  
  case 0: 
    return(lambda2*(lnT-lntau+1.5));
  
  case 1:
    return(lambda2*(
             lnT-lntau+1.5
             -.5*(lag+1)*(lag+1)*log(lag+1)
             ));

  default:  
    if (lag>=exp(lnT)) return(0);
    return(lambda2*(
             lnT-lntau+1.5+lag*lag*log(lag)
             -.5*(lag-1)*(lag-1)*log(lag-1)
             -.5*(lag+1)*(lag+1)*log(lag+1)
           ));
  }         
}

/* log(mrw) theoretical auto-correlation */
// ????? A CHANGER
// LWFLOAT CovLnX (int n, int m, LWFLOAT lambda2, LWFLOAT T)
static LWFLOAT AutoCovarianceLn(SIGNAL params,int lag)
{
  if (lag != 0) return(AutoCovarianceOm(params,lag));
  else return(AutoCovarianceOm(params,lag)+VarianceLnEps(params));
}

/* 
 * log(mrw) theoretical gradient autocorrelation  according to parameter number i
 */
static LWFLOAT DAutoCovarianceLn(SIGNAL params, int lag, int i)
{
  LWFLOAT lambda2 = params->Y[GMMP_Lambda2];
  LWFLOAT lnT = params->Y[GMMP_LnT];
  
  switch (i) {
  
  case GMMP_Sigma2:
    return(0);

  case GMMP_Lambda2:
      switch(lag) {
    case 0: return(lnT-lntau+1.5);
    default: return(AutoCovarianceLn(params,lag)/lambda2);
    }
    
  case GMMP_LnT: // ???? A CHANGER Pas tout  fait 
    if (lag>exp(lnT)) return(0);
    return(lambda2);

  default: Errorf("Unexpected error");

  }
}

/* log(mrw) theoretical mean */
 LWFLOAT MeanLn(SIGNAL params)
{
  LWFLOAT lambda2 = params->Y[GMMP_Lambda2];
  LWFLOAT lnT = params->Y[GMMP_LnT];
  LWFLOAT lnsigma,sigma2;

  lnsigma = params->Y[GMMP_Sigma2];

  // ?????
  // A CHANGER dans fichier mrw_predic.h
  // MEAN_LOG_GAUSS  = -.5*(log(2)+EULER)
  // et mettre fonction  LWFLOAT MeanLnX(LWFLOAT m, LWFLOAT lambda2, LWFLOAT sigma2, LWFLOAT T);
  return(.5*lntau-.5*(log(2)+EULER)+lnsigma-lambda2*(lnT+1.5-lntau));
}


/* 
 * log(mrw) theoretical gradient mean according to parameter number i
 */
static LWFLOAT DMeanLn(SIGNAL params, int i)
{
  LWFLOAT lambda2 = params->Y[GMMP_Lambda2];
  LWFLOAT lnT = params->Y[GMMP_LnT];

  switch (i) {
  
  case GMMP_Sigma2:
    return(1); 
  	
  case GMMP_Lambda2:
    return(-(lnT+1.5-lntau));
    
  case GMMP_LnT:
    return(-lambda2);

  default: Errorf("Unexpected error");
    
  }
}

/* mrw Var2 */
 LWFLOAT Var2(SIGNAL params)
{
  LWFLOAT lnsigma  = params->Y[GMMP_Sigma2];  
  LWFLOAT sigma2 = exp(2*lnsigma);
  
  return(sigma2*exp(lntau));
}

/* 
 * mrw  theoretical gradient Var2 according to parameter number i
 */
static LWFLOAT DVar2(SIGNAL params, int i)
{
  LWFLOAT lnsigma  = params->Y[GMMP_Sigma2];  

  switch (i) {
  
  case GMMP_Sigma2:
//  	return(1);
    return(2*exp(2*lnsigma)*exp(lntau));
  	
  case GMMP_Lambda2:
    return(0);
    
  case GMMP_LnT:
    return(0);

  default: Errorf("Unexpected error");
    
  }
}

/* 
 * returns the coefficient (i,j) of the inverse weight GMM matrix (generally the asymptotic covariance matrix of the GMMVector) 
 */
static LWFLOAT GMMMatrix(int i, int j, SIGNAL params)
{
  LWFLOAT lnsigma  = params->Y[GMMP_Sigma2];  
  LWFLOAT sigma2 = exp(2*lnsigma);
  LWFLOAT lnT = params->Y[GMMP_LnT];
  int lagMax = (int) (exp(lnT)+1);
  LWFLOAT f,r;
  int lag1,lag2,l;
  
  f = 0;
      
  /*  Var2 X ... */
  if (i==GMMV_Var2 || j==GMMV_Var2) {
    if (i == GMMV_Var2) {
      i = j;
      j = GMMV_Var2;
    }
    
    /*  Var2 X Var2 */
    if (i == GMMV_Var2) {
      f = 0;
      for (l=-lagMax;l<=lagMax;l++) {
        f+= (exp(4*AutoCovarianceOm(params,l))-1);
      }
      f+=2*exp(4*AutoCovarianceOm(params,0));
      f*= sigma2*sigma2*exp(2*lntau);
      return(f);
    }
    
    /* Var2 X VarLn */
    if (i == GMMV_VarLn) {
      f = 0;
      for (l=-lagMax;l<=lagMax;l++) {
        r = AutoCovarianceOm(params,l);
        f+= r*r;
      }
      f*=4;
      f-=VarianceLnEps(params);
      f+= 4*AutoCovarianceOm(params,0)*MeanEps2lnEps(params);
      f+= MeanEps2lnEps2(params);
    
      f*=sigma2*exp(lntau);
      return(f);
    }
    
    /* Var2 X AutoCorLn */
    if (i >= GMMV_AutoCorLn) {
      if (i-GMMV_AutoCorLn >=  theLags->size) Errorf("GMMMatrix(): Bad value for i");
      lag1 = (int) theLags->Y[i-GMMV_AutoCorLn];
      f = 0;
      for (l=-lagMax;l<=lagMax;l++) {
        f+= AutoCovarianceOm(params,l)*AutoCovarianceOm(params,lag1-l);
      }
      f+=MeanEps2lnEps(params)*AutoCovarianceOm(params,lag1);
      f*=4*sigma2*exp(lntau);
      return(f);
    }    
  }
  
  /* AutoCorLn X AutoCorLn */
  if (i >= GMMV_AutoCorLn && j >= GMMV_AutoCorLn) {
    if (i-GMMV_AutoCorLn >=  theLags->size || j-GMMV_AutoCorLn >=  theLags->size) Errorf("GMMMatrix(): Bad value for i or j");
    lag1 = (int) theLags->Y[i-GMMV_AutoCorLn];
    lag2 = (int) theLags->Y[j-GMMV_AutoCorLn];
    for (l=-lagMax;l<=lagMax+lag1;l++) {
      f+= AutoCovarianceOm(params,l)*AutoCovarianceOm(params,l+lag2-lag1)+
          AutoCovarianceOm(params,l-lag1)*AutoCovarianceOm(params,l+lag2);
    }
    f+=2*VarianceLnEps(params)*(AutoCovarianceOm(params,lag1-lag2)+AutoCovarianceOm(params,lag1+lag2));
    if (i==j) f+= VarianceLnEps(params)*VarianceLnEps(params);
    return(f);
  }
      
  /* AutoCorLn X VarLn */
  if (i >= GMMV_AutoCorLn && j == GMMV_VarLn || j >= GMMV_AutoCorLn && i == GMMV_VarLn) {
    if (i >= GMMV_AutoCorLn) lag1 = (int) theLags->Y[i-GMMV_AutoCorLn];
    else lag1 = (int) theLags->Y[j-GMMV_AutoCorLn];
    for (l=-lagMax;l<=lagMax;l++) {
      f+= 2*AutoCovarianceOm(params,l)*AutoCovarianceOm(params,lag1-l);
    }
    f+=4*AutoCovarianceOm(params,lag1)*VarianceLnEps(params);
    return(f);
  }
  
  /* VarLn X VarLn */
  if (i == GMMV_VarLn && j == GMMV_VarLn) {
    for (l=-lagMax;l<=lagMax;l++) {
      r = AutoCovarianceOm(params,l);
      f+= 2*r*r;
    }
    r = VarianceLnEps(params);
    f+=4*AutoCovarianceOm(params,0)*r-r*r+ M4LnEps(params);
    return(f);  
  }

  /* MeanLn X AutoCorLn */
/*  if (i == GMMV_MeanLn && j >= GMMV_AutoCorLn || j == GMMV_MeanLn && i >= GMMV_AutoCorLn) {
    return(0);
  }
*/  
  /* MeanLn X VarLn */
/*  if (i == GMMV_VarLn && j == GMMV_MeanLn || j == GMMV_VarLn && i == GMMV_MeanLn) {
    return(M3LnEps(params));
  }
*/
  /* MeanLn X MeanLn */
/*  if (i == GMMV_MeanLn && j == GMMV_MeanLn) {
    for (l=-lagMax;l<=lagMax;l++) {
      f += AutoCovarianceOm(params,l);
    }
    f+=VarianceLnEps(params);
    return(f);
  }
*/

  Errorf("GMMMatrix : unknown covariance %d X %d",i,j);  
}

/* 
 * returns the coefficient (i,j) of the inverse weight GMM matrix (generally the asymptotic covariance matrix of the GMMVector) 
 */
static LWFLOAT GMMMatrix1(int i, int j, SIGNAL params)
{
  LWFLOAT lnsigma  = params->Y[GMMP_Sigma2];  
  LWFLOAT sigma2 = exp(2*lnsigma);
  LWFLOAT lnT = params->Y[GMMP_LnT];
  int lagMax = (int) (exp(lnT)+1);
  LWFLOAT f,r;
  int lag1,lag2,l;
  
  f = 0;
      
  /*  Var2 X ... */
  if (i==GMMV_Var2 || j==GMMV_Var2) {
    if (i == GMMV_Var2) {
      i = j;
      j = GMMV_Var2;
    }
    
    /*  Var2 X Var2 */
    if (i == GMMV_Var2) {
      f = sigma2*sigma2*exp(2*lntau)*(3*exp(4*AutoCovarianceOm(params,0))-1);
      return(f);
    }
    
    /* Var2 X VarLn */
    if (i == GMMV_VarLn) {
      f = 0;
      r = AutoCovarianceOm(params,0);
      f+= r*r;
      f*=4;
      f-=VarianceLnEps(params);
      f+= 4*AutoCovarianceOm(params,0)*MeanEps2lnEps(params);
      f+= MeanEps2lnEps2(params);
      f*=sigma2*exp(lntau);
      return(f);
    }
    
    /* Var2 X AutoCorLn */
    if (i >= GMMV_AutoCorLn) {
      if (i-GMMV_AutoCorLn >=  theLags->size) Errorf("GMMMatrix(): Bad value for i");
      lag1 = (int) theLags->Y[i-GMMV_AutoCorLn];
      f = 0;
      f+= 4*AutoCovarianceOm(params,0)*AutoCovarianceOm(params,lag1);
      f+=2*MeanEps2lnEps(params)*AutoCovarianceOm(params,lag1);
      f*=sigma2*exp(lntau);
      return(f);
    }    
  }
  
  /* AutoCorLn X AutoCorLn */
  if (i >= GMMV_AutoCorLn && j >= GMMV_AutoCorLn) {
    if (i-GMMV_AutoCorLn >=  theLags->size || j-GMMV_AutoCorLn >=  theLags->size) Errorf("GMMMatrix(): Bad value for i or j");
    lag1 = (int) theLags->Y[i-GMMV_AutoCorLn];
    lag2 = (int) theLags->Y[j-GMMV_AutoCorLn];
    f = 0;
    f+= AutoCovarianceOm(params,0)*AutoCovarianceOm(params,lag2-lag1)+
        AutoCovarianceOm(params,-lag1)*AutoCovarianceOm(params,lag2);
    if (i == j) f+= VarianceLnEps(params)*VarianceLnEps(params)+2*VarianceLnEps(params)*AutoCovarianceOm(params,0);
    else f+= VarianceLnEps(params)*AutoCovarianceOm(params,lag1-lag2);
    return(f);
  }
      
  /* AutoCorLn X VarLn */
  if (i >= GMMV_AutoCorLn && j == GMMV_VarLn || j >= GMMV_AutoCorLn && i == GMMV_VarLn) {
    if (i >= GMMV_AutoCorLn) lag1 = (int) theLags->Y[i-GMMV_AutoCorLn];
    else lag1 = (int) theLags->Y[j-GMMV_AutoCorLn];
    f = 0;
    f+= 2*AutoCovarianceOm(params,0)*AutoCovarianceOm(params,lag1);
    f+=2*AutoCovarianceOm(params,lag1)*VarianceLnEps(params);
    return(f);
  }
  
  /* VarLn X VarLn */
  if (i == GMMV_VarLn && j == GMMV_VarLn) {
    f = 0;
    r = AutoCovarianceOm(params,0);
    f+= 2*r*r;
    r = VarianceLnEps(params);
    f+=4*AutoCovarianceOm(params,0)*r-r*r+M4LnEps(params);
    return(f);  
  }

  /* MeanLn X AutoCorLn */
/*  if (i == GMMV_MeanLn && j >= GMMV_AutoCorLn || j == GMMV_MeanLn && i >= GMMV_AutoCorLn) {
    return(0);
  }
*/  
  /* MeanLn X VarLn */
/*  if (i == GMMV_VarLn && j == GMMV_MeanLn || j == GMMV_VarLn && i == GMMV_MeanLn) {
    return(M3LnEps(params));
  }
*/
  /* MeanLn X MeanLn */
/*  if (i == GMMV_MeanLn && j == GMMV_MeanLn) {
    for (l=-lagMax;l<=lagMax;l++) {
      f += AutoCovarianceOm(params,l);
    }
    f+=VarianceLnEps(params);
    return(f);
  }
*/

  Errorf("GMMMatrix : unknown covariance %d X %d",i,j);  
}


/* 
 * returns the nth value of the GMM vector
 */
static LWFLOAT GMMVector(int n, SIGNAL params)
{
  LWFLOAT lambda2 = params->Y[GMMP_Lambda2];
  LWFLOAT lnT = params->Y[GMMP_LnT];
  LWFLOAT meanLog;
  int lag,lagIndex,meanIndex;
  
  switch (n) {

  case GMMV_Var2: 
    return (var2-Var2(params));
    
/*  case GMMV_MeanLn: 
    return(m->Y[0]-MeanLn(params));
*/
  
  case GMMV_VarLn: 
    meanLog = MeanLn(params);
    return(m2->Y[0]-2*m->Y[0]*meanLog+meanLog*meanLog-AutoCovarianceLn(params,0));
  
  default: 
    if (n>=GMMV_AutoCorLn) {
      lagIndex = n-GMMV_AutoCorLn;
      meanIndex = lagIndex+1;
      lag =(int) theLags->Y[lagIndex];
      meanLog = MeanLn(params);
      return(m2->Y[meanIndex]-meanLog*(m->Y[0]+m->Y[meanIndex])+meanLog*meanLog-AutoCovarianceLn(params,lag));
    }
    Errorf("Unexpected error");
  }
}

/* 
 * returns the nth value of the gradient of the GMM vector according to param number i 
 * (This is g_n(theta))
 */
static LWFLOAT DGMMVector(int n, int i, SIGNAL params)
{
  LWFLOAT lambda2 = params->Y[GMMP_Lambda2];
  LWFLOAT lnT = params->Y[GMMP_LnT];
  int lag,lagIndex,meanIndex;
  
  switch (n) {
  
  case GMMV_Var2: 
    return (-DVar2(params,i));
    
/*  case GMMV_MeanLn:
    return(-DMeanLn(params,i));
*/
    
  case GMMV_VarLn: 
    return(-2*DMeanLn(params,i)*(m->Y[0]-MeanLn(params)) - DAutoCovarianceLn(params,0,i));

  default: 
    if (n>=GMMV_AutoCorLn) {
      lagIndex = n-GMMV_AutoCorLn;
      meanIndex = lagIndex+1;
      lag = (int) theLags->Y[lagIndex];
      return(-DMeanLn(params,i)*(m->Y[0]+m->Y[meanIndex]-2*MeanLn(params))-DAutoCovarianceLn(params,lag,i));
    }
    Errorf("Unexpected error");
  }
}


/* 
 * returns the nth value of the mean of the gradient of the GMM vector according to param number i 
 * (This is G(theta))
 */
static LWFLOAT MDGMMVector(int n, int i, SIGNAL params)
{
  LWFLOAT lambda2 = params->Y[GMMP_Lambda2];
  LWFLOAT lnT = params->Y[GMMP_LnT];
  int lag,lagIndex,meanIndex;
  
  switch (n) {
  
  case GMMV_Var2: 
    return (-DVar2(params,i));
    
/*  case GMMV_MeanLn:
    return(-DMeanLn(params,i));
*/
    
  case GMMV_VarLn: 
    return(- DAutoCovarianceLn(params,0,i));

  default: 
    if (n>=GMMV_AutoCorLn) {
      lagIndex = n-GMMV_AutoCorLn;
      meanIndex = lagIndex+1;
      lag = (int) theLags->Y[lagIndex];
      return(-DAutoCovarianceLn(params,lag,i));
    }
    Errorf("Unexpected error");
  }
}


static void PrintParam(SIGNAL param)
{
  int i;
  LWFLOAT lambda2 = param->Y[GMMP_Lambda2];
  LWFLOAT lnT = param->Y[GMMP_LnT];
  LWFLOAT lnsigma = param->Y[GMMP_Sigma2];
  
  for (i=0;i<param->size;i++) {
  
  switch (i) {
  
  case GMMP_Sigma2:
    Printf("Sigma=%g   ",exp(lnsigma));
    break;
  	
  case GMMP_Lambda2:
    Printf("Lambda2=%g   ",lambda2);
    break;
    
  case GMMP_LnT:
    Printf("T=%g   ",exp(lnT));
    break;

  default: Errorf("Unexpected error");
    
  }
  }
}




/* 
 * Performs some init to be run before GMM is performed:
 *
 *   - Init the lags, performs allocation
 *   - computation of var2 = <X_{t}^2>
 *   - computation of m[0] = <ln(X_{t})> and m[n+1] = <ln(X_{t-lag[n]})>
 *   - computation of m2[0] = <ln(X_{t})^2> and m2[n+1] = <ln(X_t)ln(X_{t-lag[n])}>
 *
 * returns the number of samples used for computing the various means
 */
static int theN = 0;
static void InitMRWStats(SIGNAL mrw, SIGNAL lags, int tau,char flagSigma)
{
  int i,j;
  int lag;
  SIGNAL x;
  LWFLOAT r;
  int N;
  
  lntau = log(tau);

  /* Compute the variance */
  var2 = 0;
  for(i=0;i<mrw->size-tau;i++) {
    r = mrw->Y[i+tau]-mrw->Y[i];
    if (i>=lags->Y[lags->size-1]) var2 += r*r;
  }
  var2 /= mrw->size-tau-lags->Y[lags->size-1];

  theN = N = (int) (mrw->size-tau-lags->Y[lags->size-1]);

  /* If it's just what we need then we return */
  if (flagSigma) return;
    
  /* 
   * Computes
   *   x := log increments signal (for now zero increments are not dealt with)
   */
  x = NewSignal();
  SizeSignal(x,mrw->size-tau,YSIG);
  for(i=0;i<mrw->size-tau;i++) {
    r = mrw->Y[i+tau]-mrw->Y[i];
    x->Y[i] = log(fabs(r));
  }
  
  /*
   * Dealing with the lags
   * We must compute all the correlation functions and all the means
   */
  if (theLags != NULL) DeleteSignal(theLags);
  theLags = NewSignal();
  CopySig(lags,theLags);
  
  if (m != NULL) DeleteSignal(m);
  m = NewSignal();
  SizeSignal(m,lags->size+1,YSIG);
  
  if (m2 != NULL) DeleteSignal(m2);
  m2 = NewSignal();
  SizeSignal(m2,lags->size+1,YSIG);
    
  for (j=0;j<=lags->size;j++) {
    m->Y[j] = m2->Y[j] = 0; 
    if (j == 0) lag = 0;
    else lag =(int)  theLags->Y[j-1];
    for(i=(int) lags->Y[lags->size-1];i<x->size;i++) {
      m2->Y[j]+=x->Y[i]*x->Y[i-lag];
      m->Y[j]+=x->Y[i-lag];
    }
    m2->Y[j]/=x->size-lags->Y[lags->size-1];
    m->Y[j]/=x->size-lags->Y[lags->size-1];
  }  
  DeleteSignal(x);
}


/*
 * Initialisation of Gmm procedure for MRW. The historical sqrt(variance) of the mrw is returned
 *
 *    <mrw> : the mrw signal
 *    <lags> : for gmm estimation the covariance of the log of the mrw signal is used. <lags> is the signal of the lags of this covariance (0 notincluded)
 *    <tau> : scale at which the gmm estimation is launched
 *    <flagSigma> : if 1 then does not really achieve the gmm initialization (the call is just made to get the variance)
 */
LWFLOAT MRWInitGmm(SIGNAL mrw, SIGNAL lags, int tau,char flagSigma)
{  
  extern void InitGMM(int N, int nMom, int nParam, LWFLOAT (*momFunc)(int n, SIGNAL params), LWFLOAT (*dmomFunc)(int nMom, int nParam, SIGNAL params), LWFLOAT (*mdmomFunc)(int nMom, int nParam, SIGNAL params), LWFLOAT (*matrixFunc)(int i, int j, SIGNAL params), void (*printParamFunc)(SIGNAL params));

  if (tau <=0) Errorf("MRWInitGmm() : bad value for tau (=%d)",tau);

  // Compute some stats
  InitMRWStats(mrw,lags,tau,flagSigma);
  
  // Gmm estimation with the previous stats
  if (!flagSigma) {
    InitGMM(theN,lags->size+GMMV_AutoCorLn,GMMP_Last,GMMVector,DGMMVector,MDGMMVector,GMMMatrix,PrintParam);  
//    Printf("sigma %g (lnsigma = %g)\n",sqrt(var2),.5*log(var2));
  }
  
  return(sqrt(var2));
}


/*
 * Command for Initialisation of Gmm procedure for MRW
 */
void C_MrwInitGmm(char **argv)
{
   SIGNAL mrw,lags;
   int tau;
   LWFLOAT f;
   char opt;
   char flagSigma;
   
   if (*argv == NULL) {
     SetResultInt(theN);
     return;
   }
   
   // Getting the mrw signal
   argv = ParseArgv(argv,tSIGNALI,&mrw,-1);
   
   // Getting the lags signal
   if (ParseFloat_(*argv,0,&f)) {
     lags = TNewSignal();
     SizeSignal(lags,1,YSIG);
     lags->Y[0] = f;
   }
   else ParseSignalI(*argv,&lags);
   argv++;
   
   // Getting tau
   argv = ParseArgv(argv,tINT,&tau,-1); 
   
   // Options 
   flagSigma = NO;
   while (opt=ParseOption(&argv)) {
     switch (opt) {   
     case 's':
       flagSigma = YES; 
       break;
     default:
       ErrorOption(opt);
     }
   }

   SetResultFloat(MRWInitGmm(mrw,lags,tau,flagSigma));
}

